import 'dart:collection';
import 'dart:io';

import 'package:d_dio/src/type/type.dart';
import 'package:flutter/foundation.dart';

class BaseReq {
  final bool needToken; //是否需要token
  final CacheReader cacheReader;

  late String language;
  late String channel;
  late String version;
  late String time;
  late String deviceId;
  late String ip;
  late int cdnVersion;
  String? token;

  int get platform => kIsWeb
      ? 1
      : Platform.isIOS
          ? 3
          : 4;

  String get system => kIsWeb
      ? 'h5'
      : Platform.isIOS
          ? 'ios'
          : 'android';

  BaseReq({this.needToken = false, required this.cacheReader}) {
    time = DateTime.now().millisecondsSinceEpoch.toString();

    language =
        cacheReader('APP-LOCALE', defaultValue: 'zh-CN')!;

    channel = cacheReader('APP-CHANNEL', defaultValue: 'unknown')!;

    version = cacheReader('APP-VERSION', defaultValue: '0.0.0')!;

    deviceId = cacheReader('DEVICE_ID', defaultValue: 'unknown')!;

    ip = cacheReader('DEVICE_IP', defaultValue: '0.0.0.0')!;

    cdnVersion = cacheReader('CDN_VERSION', defaultValue: -1)!;

    if (needToken) token = cacheReader('user_token', defaultValue: '')!;
  }

  Map<String, dynamic> toJson() {
    final data = SplayTreeMap<String, dynamic>();
    data['language'] = language;
    data['platform'] = platform;
    data['system'] = system;
    data['channel'] = channel;
    data['version'] = version;
    data['time'] = time;
    data['device_id'] = deviceId;
    data['ip'] = ip;
    data['cdn_version'] = cdnVersion;
    if (needToken) data['token'] = token;

    return data;
  }
}
