import 'dart:async';
import 'dart:io';

import 'package:d_dio/src/const/consts.dart';
import 'package:d_dio/src/exception/api_exception.dart';
import 'package:d_dio/src/type/type.dart';
import 'package:d_dio/src/utils/log.dart';
import 'package:dio/dio.dart';

/// 返回值泛型转换
/// 经过error拦截器的处理
/// 此处的response的data均为success为true的Json数据
FutureOr<T> onResponse<T>(Response response, JsonConvert jsonConvert) {
  dynamic data = response.data['data'];
  if (T.toString() == 'String' ||
      T.toString() == 'bool' ||
      T.toString() == 'num') {
    return data as T;
  }
  return ('dynamic' == T.toString() || data == null)
      ? data
      : jsonConvert<T>(response.data['data']);
}

void onError(e) {
  if (e is DioError) {
    e = _handleDioError(e);
  } else {
    logger.e('Http存在非DioError异常: $e');
  }
  throw e;
}

Exception _handleDioError(DioError e) {
  switch (e.type) {

    /// 超时错误
    case DioErrorType.connectTimeout:
    case DioErrorType.sendTimeout:
    case DioErrorType.receiveTimeout:
      return APIException(kNetworkTimeoutCode, '请求超时');

    /// 服务异常
    case DioErrorType.response:
      return APIException(
          kServerErrorCode, 'Server Error, Code ${e.response?.statusCode}.');

    /// 请求取消
    case DioErrorType.cancel:
      return APIException(kUnknownErrorCode, '请求已取消');

    /// API异常 或 网络异常
    case DioErrorType.other:
      if (e.error is APIException) {
        ///ErrorInterceptor内的自定义异常，直接抛出
        return e.error;
      } else if (e.error is SocketException) {
        return APIException(kNetworkErrorCode, '请检查网络');
      } else if (e.error is NoSuchMethodError) {
        return APIException(kUnknownErrorCode, '请求错误');
      } else if (e.error is HandshakeException) {
        return APIException(kUnknownErrorCode, '请求失败');
      }
      logger.e('Http存在未处理异常: $e');
      return APIException(kUnknownErrorCode, e.message);
  }
}
