import 'package:d_dio/src/const/consts.dart';
import 'package:d_dio/src/entity/base_resp.dart';
import 'package:d_dio/src/exception/api_exception.dart';
import 'package:d_dio/src/type/type.dart';
import 'package:d_dio/src/utils/log.dart';
import 'package:dio/dio.dart';

class ErrorInterceptor extends InterceptorsWrapper {
  final ErrorHandler? errorHandler;

  ErrorInterceptor({this.errorHandler});

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) {
    if (response.data is Map<String, dynamic>) {
      BaseRespEntity resp = BaseRespEntity.fromJson(response.data);
      if (!resp.success) {
        logger.d(
            'API error: code:${resp.code} msg:${resp.msg} data:${resp.data}');
        errorHandler?.call(resp);
        throw APIException(resp.code, resp.msg, resp.data);
      }
    } else if (response.data is ResponseBody) {
      logger.d('API new response type: ${response.data.runtimeType}');
    } else {
      throw APIException(
          response.statusCode ?? kUnknownErrorCode, response.data);
    }
    super.onResponse(response, handler);
  }
}
