import 'package:d_dio/src/const/consts.dart';
import 'package:d_dio/src/utils/interceptor_utils.dart';
import 'package:dio/dio.dart';
import 'package:http_parser/http_parser.dart';

class FormDataInterceptor extends InterceptorsWrapper {
  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) {
    final data = InterceptorUtils.getAllParams(options);

    bool useFormData = InterceptorUtils.getCustomParams('_formData', options);
    if (useFormData) {
      List<MapEntry<String, MultipartFile>> files = [];

      ///读取并构建file
      data.forEach((key, value) {
        if (key.startsWith(kFilePrefix)) {
          files.add(
              _buildMapEntryFile(key.substring(kFilePrefix.length), value));
        }
      });

      ///移除旧的file占位参数
      data.removeWhere((key, value) => key.startsWith(kFilePrefix));

      ///构建FormData
      FormData formData = FormData.fromMap(data);

      ///向FormData添加文件
      formData.files.addAll(files);

      ///赋值请求参数
      options.data = formData;
    }
    super.onRequest(options, handler);
  }

  MapEntry<String, MultipartFile> _buildMapEntryFile(
      String fileKey, String filePath) {
    String fileName =
        filePath.substring(filePath.lastIndexOf("/") + 1, filePath.length);
    String fileSuffix =
        fileName.substring(fileName.lastIndexOf(".") + 1, fileName.length);

    return MapEntry(
        fileKey,
        MultipartFile.fromFileSync(filePath,
            filename: fileName, contentType: MediaType("image", fileSuffix)));
  }
}
