import 'package:d_dio/src/exception/api_exception.dart';
import 'package:d_dio/src/http.dart';
import 'package:d_dio/src/type/type.dart';
import 'package:d_dio/src/utils/log.dart';
import 'package:dio/dio.dart';

class LogsInterceptor extends InterceptorsWrapper {
  final bool isLog;

  final CacheSaver? cacheSaver;
  final Map<String, int> reqStartTimes = {};

  LogsInterceptor({this.isLog = false, this.cacheSaver});

  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) {
    if (isLog) {
      logger.d('请求${options.method}: ${options.baseUrl}${options.path}' +
          '\n请求参数: ${options.data ?? options.queryParameters.toString()}' +
          '\n请求Header: ${options.headers.toString()}' +
          '\n请求时间: ${DateTime.now()}');
    }
    reqStartTimes[options.path] = DateTime.now().millisecondsSinceEpoch;

    super.onRequest(options, handler);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) {
    if (isLog) {
      logger.d('Url:${response.realUri}' +
          '\n返回参数: ${response.toString()}' +
          // '\n返回Header: ${response.headers.toString()}' +
          '\n返回时间: ${DateTime.now()}');
    }
    final path = response.requestOptions.path;
    final period = DateTime.now().millisecondsSinceEpoch - reqStartTimes[path]!;
    cacheSaver?.call('dio_timing_$path', period);

    super.onResponse(response, handler);
  }

  @override
  void onError(DioError err, ErrorInterceptorHandler handler) {
    var options = err.requestOptions;
    var error = err.error;
    if (error is APIException) {
      error.message;
    }
    if (isLog) {
      logger.e('请求出错: ${error.runtimeType} ${error.toString()}' +
          '\n${options.method}: ${options.baseUrl}${options.path}' +
          '''\n请求参数: ${options.data ?? options.queryParameters.toString()}''' +
          '\n请求Header: ${options.headers.toString()}' +
          '\n响应体: ${err.response.toString()}');
    }
    super.onError(err, handler);
  }
}
