import 'dart:collection';

import 'package:d_dio/src/const/consts.dart';
import 'package:d_dio/src/utils/encrypt_utils.dart';
import 'package:d_dio/src/utils/interceptor_utils.dart';
import 'package:d_dio/src/utils/log.dart';
import 'package:dio/dio.dart';

class SignInterceptor extends InterceptorsWrapper {
  final String secret;
  final String newSecret;

  SignInterceptor({required this.secret, required this.newSecret});

  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) {
    try {
      Map<String, dynamic> data = InterceptorUtils.getAllParams(options);

      ///使用formdata时 file部分不参与加密
      bool useFormData = InterceptorUtils.getCustomParams('_formData', options,
          removeAfterRead: false);
      if (options.path.contains('mercury-web-api/')) {
        data.remove('device_id'); /// device_id中的特殊字符使前后端生成sign不一样 临时处理
        options.headers.addAll({
          'signature': genNewSign(data),
          'TimeStamp': DateTime.now().millisecondsSinceEpoch ~/ 1000
        });
      } else {
        data['sign'] =
            genSign(data, prefixIgnore: useFormData ? kFilePrefix : null);
      }
    } catch (e) {
      logger.d("SignInterceptor:$e");
    }
    super.onRequest(options, handler);
  }

  /// 参数排序生成sign 前缀为prefixRemove的字段不参与生成sign
  String genSign(Map<String, dynamic> map, {String? prefixIgnore}) {
    final mapTemp = <String, dynamic>{}..addAll(map);
    if (prefixIgnore != null) {
      mapTemp.removeWhere((key, value) => key.startsWith(prefixIgnore));
    }
    final stMap = SplayTreeMap<String, dynamic>()..addAll(mapTemp);
    final buffer = StringBuffer();
    stMap.forEach((key, value) {
      buffer
        ..write(key)
        ..write(value);
    });
    buffer.write(secret);
    return EncryptUtils.md5Encrypt(buffer.toString());
  }

  /// 新sign
  String genNewSign(Map<String, dynamic> map, {String? prefixIgnore}) {
    final mapTemp = <String, dynamic>{}..addAll(map);
    if (prefixIgnore != null) {
      mapTemp.removeWhere((key, value) => key.startsWith(prefixIgnore));
    }
    final stMap = SplayTreeMap<String, dynamic>()..addAll(mapTemp);
    StringBuffer buffer = StringBuffer();
    stMap.forEach((key, value) {
      if (buffer.isNotEmpty) {
        buffer.write('&');
      }
      buffer
        ..write(key)
        ..write('=')
        ..write(value);
    });
    buffer.write('${DateTime.now().millisecondsSinceEpoch ~/ 1000}');

    return EncryptUtils.hmacSha256Encrypt(buffer.toString(), newSecret);
  }
}
